@@ -1884,29 +1884,23 @@ enum op_type {
 
 /* perform bitwise in-place operation */
 static int
-bitwise(bitarrayobject *self, PyObject *arg, enum op_type oper)
+bitwise(bitarrayobject *self, bitarrayobject *other, enum op_type oper)
 {
-    bitarrayobject *other;
+    const Py_ssize_t nbytes = Py_SIZE(self);
     Py_ssize_t i = 0;
 
-    if (!bitarray_Check(arg)) {
-        PyErr_SetString(PyExc_TypeError,
-                        "bitarray expected for bitwise operation");
-        return -1;
-    }
-    other = (bitarrayobject *) arg;
+    // assert(self->nbits == other->nbits);
+    // assert(self->endian == other->endian);
+
     if (self->nbits != other->nbits || self->endian != other->endian) {
         PyErr_SetString(PyExc_ValueError,
                "bitarrays of equal length and endianness expected");
         return -1;
     }
-    setunused(self);
-    setunused(other);
-    Py_ssize_t size = Py_SIZE(self);
 
 #if HAS_VECTORS
     #define BITWISE_VECTOR_OP(OP)                                     \
-   for (; (Py_ssize_t)(i + sizeof(vec)) < size; i += sizeof(vec))  \
+   for (; (Py_ssize_t)(i + sizeof(vec)) < nbytes; i += sizeof(vec))   \
      vector_op(self->ob_item + i, other->ob_item + i, OP)
 #else
 #define BITWISE_VECTOR_OP(OP)
@@ -1915,17 +1909,17 @@ bitwise(bitarrayobject *self, PyObject *arg, enum op_type oper)
     switch (oper) {
     case OP_and:
         BITWISE_VECTOR_OP(&)
-        for (; i < size; ++i)
+        for (; i < nbytes; ++i)
             self->ob_item[i] &= other->ob_item[i];
         break;
     case OP_or:
         BITWISE_VECTOR_OP(|)
-        for (; i < size; ++i)
+        for (; i < nbytes; ++i)
             self->ob_item[i] |= other->ob_item[i];
         break;
     case OP_xor:
         BITWISE_VECTOR_OP(^)
-        for (; i < size; ++i)
+        for (; i < nbytes; ++i)
             self->ob_item[i] ^= other->ob_item[i];
         break;
     default:                    /* cannot happen */
@@ -1935,40 +1929,73 @@ bitwise(bitarrayobject *self, PyObject *arg, enum op_type oper)
 #undef VECTOR_BITWISE_OP
 }
 
-#define BITWISE_FUNC(oper)  \
+/* return 0 if both a and b are bitarray objects, -1 on error */
+static int
+bitwise_check(PyObject *a, PyObject *b, const char *ostr)
+{
+    if (!bitarray_Check(a) || !bitarray_Check(b)) {
+        PyErr_Format(PyExc_TypeError,
+                     "unsupported operand type(s) for %s: '%s' and '%s'",
+                     ostr, Py_TYPE(a)->tp_name, Py_TYPE(b)->tp_name);
+        return -1;
+    }
+#define aa  ((bitarrayobject *) a)
+#define bb  ((bitarrayobject *) b)
+    if (aa->nbits != bb->nbits) {
+        PyErr_Format(PyExc_ValueError,
+                     "bitarrays of equal length expected for '%s'", ostr);
+        return -1;
+    }
+    if (aa->endian != bb->endian) {
+        PyErr_Format(PyExc_ValueError,
+                     "bitarrays of equal endianness expected for '%s'", ostr);
+        return -1;
+    }
+#undef aa
+#undef bb
+    return 0;
+}
+
+#define BITWISE_FUNC(oper, ostr)                                    \
 static PyObject *                                                   \
-bitarray_ ## oper (bitarrayobject *self, PyObject *other)           \
+bitarray_ ## oper (PyObject *self, PyObject *other)                 \
 {                                                                   \
     PyObject *res;                                                  \
                                                                     \
-    res = bitarray_copy(self);                                      \
+    if (bitwise_check(self, other, ostr) < 0)                       \
+        return NULL;                                                \
+    res = bitarray_copy((bitarrayobject *) self);                   \
     if (res == NULL)                                                \
         return NULL;                                                \
-    if (bitwise((bitarrayobject *) res, other, OP_ ## oper) < 0) {  \
+    if (bitwise((bitarrayobject *) res,                             \
+                (bitarrayobject *) other, OP_ ## oper) < 0) {       \
         Py_DECREF(res);                                             \
         return NULL;                                                \
     }                                                               \
     return res;                                                     \
 }
 
-BITWISE_FUNC(and)               /* bitarray_and */
-BITWISE_FUNC(or)                /* bitarray_or  */
-BITWISE_FUNC(xor)               /* bitarray_xor */
+BITWISE_FUNC(and, "&")               /* bitarray_and */
+BITWISE_FUNC(or,  "|")               /* bitarray_or  */
+BITWISE_FUNC(xor, "^")               /* bitarray_xor */
 
 
-#define BITWISE_IFUNC(oper)  \
+#define BITWISE_IFUNC(oper, ostr)                            \
 static PyObject *                                            \
-bitarray_i ## oper (bitarrayobject *self, PyObject *other)   \
+bitarray_i ## oper (PyObject *self, PyObject *other)         \
 {                                                            \
-    if (bitwise(self, other, OP_ ## oper) < 0)               \
+    if (bitwise_check(self, other, ostr) < 0)                \
+        return NULL;                                         \
+    if (bitwise((bitarrayobject *) self,                     \
+                (bitarrayobject *) other, OP_ ## oper) < 0)  \
         return NULL;                                         \
     Py_INCREF(self);                                         \
     return (PyObject *) self;                                \
 }
 
-BITWISE_IFUNC(and)              /* bitarray_iand */
-BITWISE_IFUNC(or)               /* bitarray_ior  */
-BITWISE_IFUNC(xor)              /* bitarray_ixor */
+BITWISE_IFUNC(and, "&=")             /* bitarray_iand */
+BITWISE_IFUNC(or,  "|=")             /* bitarray_ior  */
+BITWISE_IFUNC(xor, "^=")             /* bitarray_ixor */
 
 #include "_bitarray_vct.c"
 
